function [dat_n] = ExtractHarmonicbyMP(dat,periods,n_harmonic)
% ExtractHarmonicbyMP  Extract harmonic components by multiphase imaging.
% Source dimensions: w,h,phase(0,90),freq,delay
% Output dimensions: w,h,phase(0,90),freq
% Parameters:
%   dat - multiphase dumpdata (5D).
%         delay times must cover exact integer periods.
%   periods - period number covered by delay times.
%   n_harmonic - order of the harmonic. default = 1(fundamental).
%
% Copyright (C) 2013-2014 Jinyu Lin, linjy02@hotmail.com
%

% initializing
% if ~exist('n_harmonic','var') || n_harmonic<1
if n_harmonic<1
    n_harmonic = 1;
end
sz = size(dat);
if length(sz)<5 % not multiphase
    if n_harmonic==1
        dat_n = dat;
    else
        dat_n = zeros(size(dat));
    end
    return;
end
w = sz(1); 
h = sz(2);
freq_len = sz(4);
K = sz(5); % phase length
if ~exist('periods','var')
    periods = 1;
end

% inner product
phs = periods*(0:K-1)/K;
% cs = cos(2*pi*phs*n_harmonic).';
% dat_n = reshape(dat,[w*h*freq_len*2,K])*cs;
% dat_n = reshape(dat_n,[w,h,2,freq_len])*2/K;
epn = exp(-2i*pi*phs*n_harmonic)';
dat_n = reshape(dat,[w*h*freq_len*2,K])*epn;
dat_n = reshape(dat_n,[w,h,2,freq_len])/K;

