function [Hc_n] = RectifyHarmonic(dat,An,phin,n_harmonic)
% RectifyHarmonic  Rectify harmonic component from ExtractHarmonicbyMP.
% Source dimensions: w,h,phase(0,90),freq
% Output dimensions: h,w,freq
% Parameters:
%   dat - 4D complex harmonic component
%   An - amplitude of the harmonic component
%   phin - phase of the harmonic component
%   n_harmonic - order of the harmonic. default = 1(fundamental).
%
% Copyright (C) 2013-2014 Jinyu Lin, linjy02@hotmail.com
%

% initializing
% if ~exist('n_harmonic','var') || n_harmonic<1
if n_harmonic<1
    n_harmonic = 1;
end
sz = size(dat);
w = sz(1); 
h = sz(2);
freq_len = sz(4);
Hc_n = zeros([h,w,freq_len]);

% converting
if length(size(An))==2 % 2D An, phin: freq,phase(0,90)
    ep1 = exp(-1i*phin(:,1));
    ep2 = exp(-1i*(phin(:,2)+pi/2));
    for i=1:freq_len
        a = dat(:,:,1,i)*ep1(i) + dat(:,:,2,i)*ep2(i);
        Hc_n(:,:,i) = a(:,end:-1:1).';
    end
else  % 4D An, phin: freq,phase(0,90),w,h
    An = permute(An,[3 4 2 1]);
    dat = dat./An;
    phin = permute(phin,[3 4 1 2]);
    ep1 = exp(-1i*phin(:,:,:,1));
    ep2 = exp(-1i*(phin(:,:,:,2)-pi/2*n_harmonic));
    for i=1:freq_len
        a = dat(:,:,1,i).*ep1(:,:,i) + dat(:,:,2,i).*ep2(:,:,i);
        Hc_n(:,:,i) = a(:,end:-1:1).';
    end
end
