% reconstruct transient

addpath('functions')

sel_data = 'mirror_2mr6'; % data name
cf_name = 'calib\\PmCF6_stripe'; % correlation function
rawpath = 'data'; % raw data path

% time step (ns)
tau0 = 20;
tau_step = 0.1;
tau_len = 600;
% tau0 = 20;
% tau_step = 0.01;
% tau_len = 3000;

%% acquaring parameters
freq1_ac = 1;  % MHz
freq2_ac = 180;
freqstep = 1;  % do NOT use other step

phs1 = 0;
phs2 = 1; % period
phs_len = 10;

w = 165;
h = 120;
shuttersz = 1;

freq = freq1_ac:freqstep:freq2_ac;
freq_len_ac = (freq2_ac-freq1_ac)/freqstep+1;
phsstep = (phs2-phs1)/phs_len;
phs = phs1 + (0:phs_len-1)*phsstep;

sel_sh = shuttersz;

takes = 1; % number of taking

%% read data
dat = zeros(w,h,2,freq_len_ac,phs_len);
outputpath = sprintf('%s\\scene_%s',rawpath,sel_data);
for i=1:takes
    dataname = sprintf('meas_data_take%.3d.dat',i-1);
    fn = sprintf('%s\\%s',outputpath,dataname);
    fprintf('Reading %s ...\n',dataname);
    dat = dat + ReadDump_Shutter(fn,phs_len,freq_len_ac,shuttersz,h,w,sel_sh);
end

%% load correlation function
load(cf_name,'A1_str','phi1_str','A3_str','phi3_str','shutter');
%   A1, phi1: Amplitudes and phases of fundamental component
%             dimensions: frequency,0/90deg,w,h,shutter.
%   shutter shutter time.

s = length(shutter);
A1 = A1_str(freq,:,:,:,s);
phi1 = phi1_str(freq,:,:,:,s);

%% select frequencies
freq1 = 5;  % MHz
freq2 = 165;  % MHz
% freq2 = 90;  % MHz
freq_len = (freq2-freq1)/freqstep+1;
freq_st = freq1 - freq1_ac;

%% reconstruct
fprintf('\nStart Running at %s.\n',datestr(now))
tic; % start timer

dat1 = RemoveHarmonicbyMP(dat,1);
Hc1 =  ComplexImageRectify(dat1,A1,phi1);
wHc1 = Hc1(:,:,freq_st+(1:freq_len));

beta = 6;
wind = kaiser(floor(freq2/freqstep)*2+1,beta);
wind = wind(end-freq_len+1:end);
for i=1:freq_len
    wHc1(:,:,i) = Hc1(:,:,i)*wind(i);
end

% computing
trans_img = TransientwithLow_90(wHc1,freq1,freqstep,tau0,tau_step,tau_len);

tElapsed = toc;
fprintf('\nRunning Time: %f\n', tElapsed)

% save(sprintf('results\\trans_%s',sel_data),'trans_img',...
%     'tau_step','tau_len','freq','freq1','freq2','freqstep','freq_len');

%% plot
x = 130;
y =60;
tr1 = permute(trans_img(y,x,:),[3 1 2]);
t = tau0 + (0:tau_len-1)*tau_step;
figure(1); 
p = plot(t,tr1);
set(p,'linewidth',2)

%% postprocess
norm_img = CompressHDR(trans_img);

%% show video
i0 = floor(6/tau_step);
i_len = floor(20/tau_step); % 20ns clip
for i=1:i_len
    figure(10)
    image(norm_img(:,:,i0+i)*200); colormap(gray(256))
    ttl = sprintf('scene %.3d',i); title(ttl)
    pause(0.5*tau_step)
end
