% reconstruct transient using harmonic components

addpath('functions')

sel_data = 'mirror_2mr6'; % data name
cf_name = 'calib\\PmCF6_stripe'; % correlation function
rawpath = 'data'; % raw data path

% time step (ns)
tau0 = 20;
tau_step = 0.1;
tau_len = 600;
% tau0 = 20;
% tau_step = 0.01;
% tau_len = 3000;

%% acquaring parameters
freq1_ac = 1;  % MHz
freq2_ac = 180;
freqstep = 1;  % do NOT use other step

phs1 = 0;
phs2 = 1; % period
phs_len = 10;

w = 165;
h = 120;
shuttersz = 1;

freq = freq1_ac:freqstep:freq2_ac;
freq_len_ac = (freq2_ac-freq1_ac)/freqstep+1;
phsstep = (phs2-phs1)/phs_len;
phs = phs1 + (0:phs_len-1)*phsstep;

sel_sh = shuttersz;

takes = 1; % number of taking

%% read data
dat = zeros(w,h,2,freq_len_ac,phs_len);
outputpath = sprintf('%s\\scene_%s',rawpath,sel_data);
for i=1:takes
    dataname = sprintf('meas_data_take%.3d.dat',i-1);
    fn = sprintf('%s\\%s',outputpath,dataname);
    fprintf('Reading %s ...\n',dataname);
    dat = dat + ReadDump_Shutter(fn,phs_len,freq_len_ac,shuttersz,h,w,sel_sh);
end

%% load correlation function
load(cf_name,'A1_str','phi1_str','A3_str','phi3_str','shutter');
%   A1, phi1: Amplitudes and phases of fundamental component
%             dimensions: frequency,0/90deg,w,h,shutter.
%   shutter shutter time.

s = length(shutter);
A1 = A1_str(freq,:,:,:,s);
phi1 = phi1_str(freq,:,:,:,s);
A3 = A3_str(freq,:,:,:,s);
phi3 = phi3_str(freq,:,:,:,s);

%% select frequencies
% fundamental components
freq1 = 5;  % MHz
freq2 = 90;  % MHz
freq_len = (freq2-freq1)/freqstep+1;
freq_st = freq1 - freq1_ac;

% harmonic components
freq1_3 = 20;  % MHz
freq2_3 = 85;  % MHz
freq_len_3 = (freq2_3-freq1_3)/freqstep+1;
freq_st_3 = freq1_3 - freq1_ac;

%% reconstruct
fprintf('\nStart Running at %s.\n',datestr(now))
tic; % start timer

% extracting components
dat_1 = ExtractHarmonicbyMP(dat,phs2,1);
dat_3 = ExtractHarmonicbyMP(dat,phs2,3);

% rectifying amplitude and phase
Hc1 =  RectifyHarmonic(dat_1,A1,phi1,1);
Hc3 =  RectifyHarmonic(dat_3,A3,phi3,3);

% interpolating harmonic data
n_harmonic = 3;
wHc_len = (freq2_3+1)*n_harmonic - freq1;
wHc = zeros(h,w,wHc_len);
for i=1:h
    for j=1:w
        a = Hc3(i,j,freq_st_3+(1:freq_len_3));
        a_intp = interp(a(:),n_harmonic);
        wHc(i,j,end-freq_len_3*n_harmonic+1:end) = ...
            permute(a_intp,[3 2 1]);
    end
end

% combining harmonic data
wHc(:,:,1:freq_len) = Hc1(:,:,freq_st+(1:freq_len));

% windowing
beta = 6;
wind = kaiser((wHc_len+freq1)*2-1,beta);
wind = wind(end-wHc_len+1:end);
for i=1:wHc_len
    wHc(:,:,i) = wHc(:,:,i)*wind(i);
end

% computing
trans_img = TransientwithLow(wHc,freq1,freqstep,tau0,tau_step,tau_len);

tElapsed = toc;
fprintf('\nRunning Time: %f\n', tElapsed)

% save(sprintf('results\\trans_%s',sel_data),'trans_img',...
%     'tau_step','tau_len','freq','freq1','freq2','freqstep','freq_len');

%% plots
x = 130;
y =60;
tr_harm = permute(trans_img(y,x,:),[3 1 2]);
t = tau0 + (0:tau_len-1)*tau_step;
figure(1); 
p = plot(t,tr_harm);
set(p,'linewidth',2)

%% postprocess
norm_img = CompressHDR(trans_img);

t0 = 200;
img1 = max(trans_img(:,:,1:t0),[],3);
img2 = max(trans_img(:,:,t0+1:end),[],3);
figure(11); imagesc(img1/4); colormap(gray(256)); axis off
figure(12); imagesc(img2); colormap(gray(256)); axis off

%% show video
i0 = floor(10/tau_step);
i_len = floor(16/tau_step); % 16ns clip
for i=1:i_len
    figure(10)
    image(norm_img(:,:,i0+i)*256); colormap(gray(256))
    ttl = sprintf('scene %.3d',i); title(ttl)
    pause(0.5*tau_step)
end
