% show amplitude of each components at a pixel

calib_raw = 'calib\\calib0';

%% acquiring parameters
freq1 = 1;  % MHz
freq2 = 180;
freqstep = 1; 
freq = freq1:freqstep:freq2;
freq_len = length(freq);

phs1 = 0;
phs2 = 2;  % periods
phs_len = 160;
phsstep = (phs2-phs1)/phs_len;
phs = phs1 + (0:phs_len-1)*phsstep;

load(calib_raw);

%% fitting
n0 = 3;
C0 = zeros(freq_len,1); % DC
ep1 = exp(-2i*pi*phs(:));
C1 = zeros(freq_len,1); % fundamental component
Cn = zeros(freq_len,n0); % harmonic components

for i = 1:freq_len
    % expanding Fourier series
    dat1 = calib0(:,1,i);
    % DC
    C0(i) = sum(dat1)/phs_len;
    % fundamental component
    C1(i) = dat1.'*ep1/phs_len;
    % harmonic components
    for ni = 1:n0
        epn = exp(-2i*pi*(ni+1)*phs(:));
        Cn(i,ni) = dat1.'*epn/phs_len;
    end
end

figure(1); 
p=plot(freq,abs([C1, Cn])/max(abs(C1)));
set(p,'linewidth',2)

